#include <Wire.h>  // binde Bibliothek Wire fuer I2C ein (LiquidCrystal_I2C braucht diese Bibliothek)
#include <LiquidCrystal_I2C.h> //LCD Bibliothek
#include "LcdKeypad.h"
#include "MenuData.h" // Enthält die Menüstrucktur
#include <Servo.h>
#include <millisDelay.h>  //Bibliothek für einfache Benutzung nicht blockierender Timer
millisDelay VentilDelay1, VentilDelay2, VentilDelay3, VentilDelay4, VentilDelay5, VentilDelay6; //Timer für die einzelnen Ventile, damit man mehrere Ventile gleichzeitig öffnen kann.
millisDelay MixenDelay; //Timer für den Mixer-Motor
millisDelay ShaServDelay; //Timer für den Servo, welcher den Shaker ausgießt
millisDelay ZuckerDelay; //Timer für den Zuckerspender
Servo ShakerServo; //Erstellt für das Programm ein Servo mit dem Namen „ShakerServo“
Servo EisServo; //Erstellt für den Eisspender ein Servo mit dem Namen „EisServo“
enum AppModeValues
{
  APP_NORMAL_MODE,
  APP_MENU_MODE,
  APP_PROCESS_MENU_CMD
};
byte appMode = APP_NORMAL_MODE;
MenuManager Menu1(sampleMenu_Root, menuCount(sampleMenu_Root));
char strbuf[LCD_COLS + 1]; // one line of lcd display
byte btn;
LiquidCrystal_I2C lcd(0x27, LCD_COLS, LCD_ROWS); // setze LCD auf 16 Zeichen und 2 Zeilen
void refreshMenuDisplay (byte refreshMode);
byte getNavAction();
//Unsere Variablen ----------------------------------------------------------------------------------------------------------
int Shakerpos = ShakerServo.read();
int ausgiessen = 0;
bool Shakerleer = false;
bool geleert = false;
bool offenes_Ventil[7];
//VentDel ist ein Pointer Array, damit alle sechs Ventile gleichzeitig und mit einer Funktion angesteuert werden können.
millisDelay (*VentDel[7]) = {NULL, &VentilDelay1, &VentilDelay2, &VentilDelay3, &VentilDelay4, &VentilDelay5, &VentilDelay6};
bool Ventiloffen[7] = {false};
bool gemixt = false;
//bool *pRezept[4] = {&Rezept1,&Rezept2,&Rezept3,&Rezept4};
bool Rezeptlauft[4] = {false};

void setup() {
  Serial.begin(9600);

  pinMode(2, OUTPUT);
  pinMode(3, OUTPUT);
  pinMode(4, OUTPUT);
  pinMode(5, OUTPUT);
  pinMode(6, OUTPUT);
  pinMode(7 , OUTPUT);
  pinMode(11, OUTPUT);
  ShakerServo.attach(9);
  EisServo.attach(10);

  lcd.init();
  lcd.backlight();

  // fall in to menu mode by default.
  appMode = APP_MENU_MODE;
  refreshMenuDisplay(REFRESH_DESCEND);
}

void loop() {
  btn = getButton();
  //Serial.println(btn);
  switch (appMode) {
    case APP_NORMAL_MODE :
      if (btn == BUTTON_UP_LONG_PRESSED) {
        appMode = APP_MENU_MODE;
        refreshMenuDisplay(REFRESH_DESCEND);
      }
      break;
    case APP_MENU_MODE : {
        byte menuMode = Menu1.handleNavigation(getNavAction, refreshMenuDisplay);
        //Serial.println("Menü");

        if (menuMode == MENU_EXIT) {
          lcd.clear();
          lcd.print("Hold UP for menu");
          appMode = APP_NORMAL_MODE;
        }
        else if (menuMode == MENU_INVOKE_ITEM) {
          appMode = APP_PROCESS_MENU_CMD;

          // Indicate selected item.
          if (Menu1.getCurrentItemCmdId()) {
            lcd.setCursor(0, 1);
            strbuf[0] = 0b01111110; // forward arrow representing input prompt.
            strbuf[1] = 0;
            lcd.print(strbuf);
          }
        }
        break;
      }
    case APP_PROCESS_MENU_CMD : {
        byte processingComplete = processMenuCommand(Menu1.getCurrentItemCmdId());
        Serial.println("prossesing");

        if (processingComplete) {
          appMode = APP_MENU_MODE;
          // clear forward arrow
          lcd.setCursor(0, 1);
          strbuf[0] = ' '; // clear forward arrow
          strbuf[1] = 0;
          lcd.print(strbuf);
        }
        break;
      }
  }

  if (ShaServDelay.justFinished()) {
    ShaServDelay.repeat();
    Shakerpos = ShakerServo.read();
    //ShakerServo.write(Shakerpos);
    //Serial.print("Shakerpos: ");
    //Serial.println(Shakerpos);

    if (Shakerpos <= 175 && ausgiessen == 0) {
      //Serial.println("Shakerleer: ");
      //Serial.println(Shakerleer);
      if (Shakerleer == false) {
        Shakerpos = 1 + Shakerpos;
        ShakerServo.write(Shakerpos);
        //Serial.print("rechnen1: ");
        //Serial.println(Shakerpos);

      } else if (Shakerleer == true) {
        Shakerpos = Shakerpos - 1;
        ShakerServo.write(Shakerpos);
        //Serial.println("rechnen2: ");
        //Serial.println(Shakerpos);
      }
    }
    if (Shakerpos > 175 && ausgiessen == 0) {
      //Serial.println("rückweg");
      Shakerleer = true;
      Shakerpos = 169;
      ShakerServo.write(Shakerpos);
      ausgiessen = 5;
    }
    if (ausgiessen > 0) {
      ausgiessen = ausgiessen - 1;
    }
    if (Shakerpos < 75 && Shakerleer == true) {
      Serial.println("fertig geshaked");
      Shakerpos = 0;
      ShakerServo.write(Shakerpos);
      Shakerleer = false;
      Shakerpos = ShakerServo.read();
      ShaServDelay.stop();
      geleert = true;
    }
  }
  for (int i; VentDel[i]->justFinished(); i++) {
    Ventil(i, 0, true);
  }
  if (MixenDelay.justFinished()) {
    digitalWrite(11, LOW);
    gemixt = true;
    Serial.println("gemixt");
  }
  if (ZuckerDelay.justFinished()) {
    digitalWrite(6, LOW);
    Serial.println("fertig gezuckert");
  }
  //  for(int i; Rezeptlauft[i] == true; i++) {
  //    pRezept[i];
  //  }
}

//----------------------------------------------------------------------
// Addition or removal of menu items in MenuData.h will require this method
// to be modified accordingly.
byte processMenuCommand(byte cmdId)
{
  byte complete = false;  // set to true when menu command processing complete.

  if (btn == BUTTON_SELECT_PRESSED)
  {
    complete = true;
  }

  switch (cmdId)
  {
    // TODO Process menu commands here:
    case mnuCmdCocktail1 :      // Cocktail 1

      break;
    case mnuCmdZutata1 :
      Serial.println("mnuCmdZutata1");
      if (btn == BUTTON_RIGHT_PRESSED) {
        if (Rezept1) {
          complete = true;
        }
      }
      break;
    case mnuCmdZutata2 :

      break;
    case mnuCmdZutata3 :

      break;
    case mnuCmdCocktail2 :      // Cocktail 2

      break;
    case mnuCmdZutatb1 :

      break;
    case mnuCmdZutatb2 :

      break;
    case mnuCmdZutatb3 :

      break;
    case mnuCmdCocktail3 :      // Cocktail 3

      break;
    case mnuCmdZutatc1 :

      break;
    case mnuCmdZutatc2 :

      break;
    case mnuCmdZutatc3 :

      break;
    case mnuCmdResetToDefaults :

      break;
  }

  return complete;
}


//----------------------------------------------------------------------
// Callback to convert button press to navigation action.
byte getNavAction() {
  byte navAction = 0;
  byte currentItemHasChildren = Menu1.currentItemHasChildren();

  if (btn == BUTTON_UP_PRESSED || btn == BUTTON_UP_LONG_PRESSED) navAction = MENU_ITEM_PREV;
  else if (btn == BUTTON_DOWN_PRESSED || btn == BUTTON_DOWN_LONG_PRESSED) navAction = MENU_ITEM_NEXT;
  else if (btn == BUTTON_SELECT_PRESSED || (btn == BUTTON_RIGHT_PRESSED && currentItemHasChildren)) navAction = MENU_ITEM_SELECT;
  else if (btn == BUTTON_LEFT_PRESSED) navAction = MENU_BACK;
  return navAction;
}

//----------------------------------------------------------------------
const char EmptyStr[] = "";

// Callback to refresh display during menu navigation, using parameter of type enum DisplayRefreshMode.
void refreshMenuDisplay (byte refreshMode)
{
  char nameBuf[LCD_COLS + 1];

  /*
    if (refreshMode == REFRESH_DESCEND || refreshMode == REFRESH_ASCEND)
    {
      byte menuCount = Menu1.getMenuItemCount();

      // uncomment below code to output menus to serial monitor
      if (Menu1.currentMenuHasParent())
      {
        Serial.print("Parent menu: ");
        Serial.println(Menu1.getParentItemName(nameBuf));
      }
      else
      {
        Serial.println("Main menu:");
      }

      for (int i=0; i<menuCount; i++)
      {
        Serial.print(Menu1.getItemName(nameBuf, i));

        if (Menu1.itemHasChildren(i))
        {
          Serial.println("->");
        }
        else
        {
          Serial.println();
        }
      }
    }
  */

  lcd.setCursor(0, 0);
  if (Menu1.currentItemHasChildren())
  {
    rpad(strbuf, Menu1.getCurrentItemName(nameBuf));
    strbuf[LCD_COLS - 1] = 0b01111110;          // Display forward arrow if this menu item has children.
    lcd.print(strbuf);
    lcd.setCursor(0, 1);
    lcd.print(rpad(strbuf, EmptyStr));          // Clear config value in display
  }
  else
  {
    byte cmdId;
    rpad(strbuf, Menu1.getCurrentItemName(nameBuf));

    if ((cmdId = Menu1.getCurrentItemCmdId()) == 0)
    {
      strbuf[LCD_COLS - 1] = 0b01111111;        // Display back arrow if this menu item ascends to parent.
      lcd.print(strbuf);
      lcd.setCursor(0, 1);
      lcd.print(rpad(strbuf, EmptyStr));        // Clear config value in display.
    }
    else
    {
      lcd.print(strbuf);
      lcd.setCursor(0, 1);
      lcd.print(" ");

      // TODO Display config value.
    }
  }
}
